@@ -180,6 +180,13 @@ SUNLinearSolver SUNLinSol_OneMklDense(N_Vector y, SUNMatrix Amat, SUNContext sun
 
   if (num_blocks > 1)
   {
+#ifdef SUNDIALS_ONEMKL_USE_GETRF_LOOP
+    LS_F_SCRATCH_SIZE(S) =
+      getrf_scratchpad_size<realtype>(*queue, // device queue
+                                      M,      // rows in A_i
+                                      N,      // columns in A_i
+                                      M);     // leading dimension
+#else
     LS_F_SCRATCH_SIZE(S) =
       getrf_batch_scratchpad_size<realtype>(*queue,      // device queue
                                             M,           // rows in A_i
@@ -188,8 +195,17 @@ SUNLinearSolver SUNLinSol_OneMklDense(N_Vector y, SUNMatrix Amat, SUNContext sun
                                             M * N,       // stride between A_i
                                             M,           // stride in P_i
                                             num_blocks); // number of blocks
+#endif
 
-#ifdef SUNDIALS_ONEMKL_USE_GETRS_BATCHED
+#ifdef SUNDIALS_ONEMKL_USE_GETRS_LOOP
+    LS_S_SCRATCH_SIZE(S) =
+      getrs_scratchpad_size<realtype>(*queue,  // device queue
+                                      oneapi::mkl::transpose::nontrans,
+                                      M,      // number of rows in A
+                                      1,      // number of right-hand sizes
+                                      M,      // leading dimension of A
+                                      M);     // leading dimension of B
+#else
     LS_S_SCRATCH_SIZE(S)=
       getrs_batch_scratchpad_size<realtype>(*queue,      // device queue
                                             oneapi::mkl::transpose::nontrans,
@@ -201,14 +217,6 @@ SUNLinearSolver SUNLinSol_OneMklDense(N_Vector y, SUNMatrix Amat, SUNContext sun
                                             M,           // leading dimension of B_i
                                             M,           // stride between B_i
                                             num_blocks); // number of blocks
-#else
-    LS_S_SCRATCH_SIZE(S) =
-      getrs_scratchpad_size<realtype>(*queue,  // device queue
-                                      oneapi::mkl::transpose::nontrans,
-                                      M,      // number of rows in A
-                                      1,      // number of right-hand sizes
-                                      M,      // leading dimension of A
-                                      M);     // leading dimension of B
 #endif
   }
   else
@@ -326,6 +334,36 @@ int SUNLinSolSetup_OneMklDense(SUNLinearSolver S, SUNMatrix A)
 
   if (num_blocks > 1)
   {
+#ifdef SUNDIALS_ONEMKL_USE_GETRF_LOOP
+    try
+    {
+      for (sunindextype i = 0; i < num_blocks; i++)
+      {
+        getrf(*queue,            // device queue
+              M,                 // number of rows
+              N,                 // number of columns
+              Adata + i * M * N, // matrix data
+              M,                 // leading dimension of A
+              pivots + i * M,    // array of pivots
+              scratchpad,        // scratchpad memory
+              scratch_size);     // scratchpad size
+      }
+    }
+    catch(oneapi::mkl::lapack::exception const& e)
+    {
+      SUNDIALS_DEBUG_ERROR("An exception occured in getrf\n");
+      if (e.info())
+      {
+        // An illegal value was providied or the scratch pad is too small
+        ier = -1;
+      }
+      else
+      {
+        // The diagonal element of some of U_i is zero
+        ier = 1;
+      }
+    }
+#else
     try
     {
       getrf_batch(*queue,         // device queue
@@ -354,6 +392,7 @@ int SUNLinSolSetup_OneMklDense(SUNLinearSolver S, SUNMatrix A)
         ier = 1;
       }
     }
+#endif
   }
   else
   {
@@ -467,7 +506,30 @@ int SUNLinSolSolve_OneMklDense(SUNLinearSolver S, SUNMatrix A, N_Vector x,
 
   if (num_blocks > 1)
   {
-#ifdef SUNDIALS_ONEMKL_USE_GETRS_BATCHED
+#ifdef SUNDIALS_ONEMKL_USE_GETRS_LOOP
+    try
+    {
+      for (sunindextype i = 0; i < num_blocks; i++)
+      {
+        getrs(*queue,            // device queue
+              oneapi::mkl::transpose::nontrans,
+              M,                 // number of rows
+              1,                 // number of right-hand sides
+              Adata + i * M * N, // factorized matrix data
+              M,                 // leading dimension of A
+              pivots,            // array of pivots
+              xdata + i * M,     // right-hand side data
+              M,                 // leading dimension of B_i
+              scratchpad,        // scratchpad memory
+              scratch_size);     // scratchpad size
+      }
+    }
+    catch(oneapi::mkl::lapack::exception const& e)
+    {
+      SUNDIALS_DEBUG_ERROR("An exception occured in getrs\n");
+      ier = -1;
+    }
+#else
     try
     {
       getrs_batch(*queue,        // device queue
@@ -491,29 +553,6 @@ int SUNLinSolSolve_OneMklDense(SUNLinearSolver S, SUNMatrix A, N_Vector x,
       SUNDIALS_DEBUG_ERROR("An exception occured in getrs_batch\n");
       ier = -1;
     }
-#else
-    try
-    {
-      for (sunindextype i = 0; i < num_blocks; i++)
-      {
-        getrs(*queue,            // device queue
-              oneapi::mkl::transpose::nontrans,
-              M,                 // number of rows
-              1,                 // number of right-hand sides
-              Adata + i * M * N, // factorized matrix data
-              M,                 // leading dimension of A
-              pivots,            // array of pivots
-              xdata + i * M,     // right-hand side data
-              M,                 // leading dimension of B_i
-              scratchpad,        // scratchpad memory
-              scratch_size);     // scratchpad size
-      }
-    }
-    catch(oneapi::mkl::lapack::exception const& e)
-    {
-      SUNDIALS_DEBUG_ERROR("An exception occured in getrs\n");
-      ier = -1;
-    }
 #endif
   }
   else
