@@ -340,7 +340,8 @@ int SUNMemoryHelper_Destroy_Hip(SUNMemoryHelper helper)
 {
   if (helper)
   {
-    free(helper->content);
+    if (helper->content) { free(helper->content); }
+    if (helper->ops) { free(helper->ops); }
     free(helper);
   }
   return 0;
